<?php
/*
 * $Horde: babel/regenerate.php,v 1.9 2005/01/03 14:34:48 jan Exp $
 *
 * Copyright 2002-2005 Jon Parise <jon@horde.org>
 * Copyright 2000, 2001 easyDNS Technologies Inc.
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/asl.php.
 */

define('BABEL_BASE', dirname(__FILE__));
require_once BABEL_BASE . '/lib/base.php';

$lang = @$_COOKIE['i18n_lang'];

$title = 'Regenerating '.$lang.' Language Files';
$subhead = '[ <b>' . makeLink('/view.php', 'i18n') . '</b> ] ';

require BABEL_TEMPLATES . '/common-header.inc';
require BABEL_TEMPLATES . '/page-header.inc';

$shown = 0;

echo <<< EOF
<table class="outline-lite" width="100%" border="0" cellspacing="1" cellpadding="0">
<tr><td>
<table width="100%" border="0" cellspacing="1" cellpadding="2">
<tr class="header">
 <th align="left">Module</th>
 <th align="left">Files</th>
 <th align="left">Strings</th>
 <th align="left">Status</th>
</tr>
EOF;

$umask = umask(0);
$lang_dir = $DIRS['lang_files'] . '/' . $lang;
if (!@is_dir($lang_dir)) {
	mkdir($lang_dir, 0777);
}
$lang_dir .= '/LC_MESSAGES';
if (!@is_dir($lang_dir)) {
	mkdir($lang_dir, 0777);
}


# get all users first

$all_users = array();
$query = 'SELECT user,name,email FROM users';
$q = $DB->query($query);
while ($r = $q->fetchRow()) {
	$all_users[$r['user']] = $r;
}



foreach ($VALID_MODULES as $module) {

	$po_file = $lang_dir . '/' . $module . '.po';
	$mo_file = $lang_dir . '/' . $module . '.mo';

	$query = sprintf("SELECT p.page_id, p.path, p.file, p.db_revision, " .
		 "s.string_id, s.line_no, s.string, s.comment, " .
		 "t.translation, t.user, UNIX_TIMESTAMP(t.ttime) AS time " .
		 "FROM translations t LEFT JOIN strings s USING (string_id) LEFT JOIN pages p USING (page_id) " .
		 "WHERE t.lang = '%s' AND p.path LIKE '%s%%' " .
		 "ORDER BY p.path, p.file, s.line_no",
		$_COOKIE['i18n_lang'],
		($CVS->cvsRoot().'/'.$module)
	);

	$q = $DB->query($query);
	$num = $q->numRows();

	$data = $errors = array();

	$data[] = '#';
	$data[] = '# easyDNS Andorra Language File';
	$data[] = '# Copyright (c) 2000, easyDNS Technologies Inc.';
	$data[] = '#';
	$data[] = '# #####################################################';
	$data[] = '#';
	$data[] = '#     LANGUAGE: ' . $lang;
	$data[] = '#       MODULE: ' . $module;
	$data[] = '# GENERATED BY: ' . $USER->name . ' <' . $USER->email . '>';
	$data[] = '#         DATE: ' . date('r');
	$data[] = '#      STRINGS: ' . $num;
	$data[] = '#';
	$data[] = '# #####################################################';
	$data[] = '';
	$data[] = '';


	if ($num>0) {

		while ($r = $q->fetchRow()) {

			$string = str_replace('\\', '\\\\', $r['string']);
			$string = str_replace('"', '\\"', $string);
			$trans = fixString($r['translation'], $lang);

			$data[] = '#       File: ' . $r['path'] . '/' . $r['file'];
			$data[] = '#   Revision: ' . $r['db_revision'];
			$data[] = '#       Line: ' . $r['line_no'];
			$data[] = '# Translator: ' . 
				$all_users[$r['user']]['name'] . ' <' .
				$all_users[$r['user']]['email'] . '>';
			$data[] = '#       Date: ' . date('r', $r['time']);
			$data[] = 'msgid  "' . $string . '"';
			$data[] = 'msgstr "' . $trans . '"';
			$data[] = '';
			$data[] = '';

		}

	}


	$status = array();

	if ($fp = @fopen($po_file,'w')) {

		$bytes = fwrite($fp, join("\n", $data) );
		fclose($fp);

		$status[] = "Wrote $bytes bytes to .po file";

		if (!@chmod($po_file, 0664)) {
			$status[] = '<b class="red">Could not chmod .po file</b>';
		}


		if (file_exists($mo_file)) {
			if (!@unlink($mo_file)) {
				$status[] = '<b class="red">Could not remove old .mo file</b>';
			}
		}

		$cmd = sprintf("%s %s -o %s",
			$BIN['msgfmt'],
			escapeshellcmd($po_file),
			escapeshellcmd($mo_file)
		);

		$out = `$cmd 2>&1`;

		if (!file_exists($mo_file)) {
			if ($num) {
				$status[] = '<b class="red">New .mo file not generated</b>';
				$errors[] = '<b>' . $cmd . '</b>';
				$errors[] = $out;
			} else {
				$status[] = 'No new .mo file required';
			}
		} else {
			$status[] = 'Wrote ' . filesize($mo_file) . ' bytes to .mo file';

			if (!@chmod($mo_file, 0664)) {
				$status[] = '<b class="red">Could not chmod .mo file</b>';
			}

		}

	} else {

		$status[] = '<b class="red">Could not open file for writing</b>';

	}

	echo '<tr valign="baseline" class="item' . ($shown%2) . '">';
	echo '<td class="row" width="10%">' . $module . '<br></td>';
	echo '<td class="row" width="50%">' . $po_file . '<br>' . $mo_file . '<br></td>';
	echo '<td class="row" width="10%">' . $num . '<br></td>';
	echo '<td class="row" width="30%">' . join('<br>',$status) . '<br></td>';


	$shown++;

}

echo <<< EOF
</table>
</td></tr>
</table>
EOF;



if (count($errors)!=0) {        
        echo <<< EOF
<br>
<table class="outline-error" width="100%" border="0" cellspacing="1" cellpadding="0">
<tr><td>
<table width="100%" border="0" cellspacing="1" cellpadding="2">
<tr class="error">
 <td>Take note of the following possible errors:<br></td>
</tr>
<tr class="error">
 <td>
EOF;

	echo "<ul>\n";
	foreach ($errors as $err) {
		echo "<li>" . nl2br($err) . "\n";
	}                       
	echo "</ul>\n";

	echo <<< EOF
 </td>
</tr>
</table>
</td></tr>
</table>

EOF;

}

require $registry->get('templates', 'horde') . '/common-footer.inc';
