<?php
/*
 * $Horde: babel/reparse.php,v 1.10 2005/01/03 14:34:48 jan Exp $
 *
 * Copyright 2002-2005 Jon Parise <jon@horde.org>
 * Copyright 2000, 2001 easyDNS Technologies Inc.
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/asl.php.
 */

define('BABEL_BASE', dirname(__FILE__));
require_once BABEL_BASE . '/lib/base.php';

$where = $CVS->cvsRoot();
$title = 'i18n Repository';
$subhead = '[ <b>' . Horde::link(Babel::url('view')) . 'i18n</a></b> ] ';

if (isset($PATH_INFO)) {
	$where .= $PATH_INFO;
	$title = 'Reparsing '.$PATH_INFO;
	$temp = explode('/', $PATH_INFO);
	$last = array_pop($temp);
	$path = '';
	foreach ($temp as $part) {
		if ($part) {
			$path .= '/' . $part;
            $subhead.= ' / <b>' . Horde::link(Babel::url('view', $path)) . $part . '</a></b>';
		}
	}

	$ext = substr( strrchr($last, '.'), 1 );
	if (in_array($ext, $conf['cvs']['extensions'])) {
		$subhead.= ' / <b>' . Horde::link(Babel::url('strings', $path . '/' . $last)) . $last . '</a></b>';
	} else {
		$subhead.= ' / <b>' . Horde::link(Babel::url('view', $path . '/' . $last)) . $last . '</a></b>';
	}
	
}

require BABEL_TEMPLATES . '/common-header.inc';
require BABEL_TEMPLATES . '/page-header.inc';
require BABEL_TEMPLATES . '/reparse/header.inc';

# get page_ids

$query = "SELECT page_id, CONCAT(path,'/',file) AS full " .
	"FROM pages WHERE file !='' HAVING FULL LIKE '" . $where . "%' ";
	"ORDER BY path, file";

$q0 = $DB->query($query);

$pages = $checksums = $ids = $added = $errors = array();
$shown = $unchanged = $duplicate = $updated = $deleted = 0;



while ($r0 = $q0->fetchRow()) {

	$page_id = $r0['page_id'];
	$pages[] = $page_id;

	# get new file revision

	$cf = new CVSLib_File($CVS,$r0['full']);
	$cf->getBrowseInfo();
	$skip = false;

	if (!strcasecmp($conf['cvs']['branch'], 'head')) {
		$key = array_search($conf['cvs']['branch'], $cf->branches, true);
		if ($key === false) {
			$new_rev = false;
			$skip = true;
		} else {
/* this is a kludge */
			$cf = new CVSLib_File($CVS,$r0['full']);
			$cf->getBrowseInfo($key);
			$new_rev = $cf->queryrevision();
		}
	} else {
		$new_rev = $cf->queryrevision();
	}

	$s = findNewStrings($r0['full'], $new_rev);

	$c = sizeof($s);
	if ($skip) {
		$c = 0;
	}

    include BABEL_TEMPLATES . '/reparse/count.inc';

	$checksums = array();

	if ($c==0) {

		$query = "SELECT * FROM strings WHERE page_id=".$page_id." ORDER BY line_no";
		$q = $DB->query($query);

		if ($q->numrows()==0) {
            include BABEL_TEMPLATES . '/reparse/empty.inc';
		} else {

			$del = array();
			while ($r=$q->fetchrow()) {
                include BABEL_TEMPLATES . '/reparse/deleted.inc';
				$del[] = $r['string_id'];
				$shown++;

			}

			$query = "DELETE FROM strings WHERE string_id IN (" . join(',',$del) . ")";
			$q = $DB->query($query);
			$deleted = $DB->affectedRows();

			$query = "DELETE FROM translations WHERE string_id IN (" . join(',',$del) . ")";
			$q = $DB->query($query);

		}

	} else {

		foreach ($s as $string) {

			if (isset($string['error'])) {
				$errors[] = $string['error'];
				continue;
			}

			if (isset($checksums[$string['checksum']])) {
                include BABEL_TEMPLATES . '/reparse/duplicate.inc';
				$duplicate++;
				$shown++;
				continue;
			}

			$checksums[$string['checksum']] = true;

			$query = sprintf("SELECT * FROM strings WHERE page_id=%d AND checksum='%s'",
				$page_id,
				$string['checksum']
			);
			$q2 = $DB->query($query);

			if ($q2->numrows()==0) {   	/* need to add */

				$sid = $DB->nextid('strings');

				$string['string_id'] = $sid;
				$string['page_id'] = $page_id;

				$ids[] = $sid;
				$checksums[$string['checksum']] = true;
				array_push($added, $sid );

				if (substr($string['string'],0,1)=='%') {
					$string['string'] = '%' . $string['string'];
				}

				$query = 'INSERT INTO strings ' . Babel::makeSQLInsert($string);
				$q2 = $DB->query($query);

                include BABEL_TEMPLATES . '/reparse/added.inc';

			} else {		/* already there, check for changes in other info */

				$r2 = $q2->fetchRow();

				$ids[] = $r2['string_id'];

				$newdata = array();
				if ($r2['line_no']!=$string['line_no']) {
					$newdata['line_no'] = $string['line_no'];
				}
				if (isset($string['comment']) && $r2['comment']!=$string['comment']) {
					$newdata['comment'] = $string['comment'];
				}

				if (sizeof($newdata)) {
					$query = sprintf('UPDATE strings SET %s WHERE string_id=%d',
						makeSQLUpdate($newdata),
						$r2['string_id']
					);
					$q2 = $DB->query($query);
					$status = 'updated';
					$updated++;
				} else {
					$status = 'unchanged';
					$unchanged++;
				}


				# if the string didn't change, just update the revision on
				# on the translation, so you don't have to look through them all
				# again.

				$query = sprintf("UPDATE translations SET db_revision='%s' WHERE string_id=%d",
					$new_rev,
					$r2['string_id']
				);
				$q2 = $DB->query($query);
                include BABEL_TEMPLATES . '/reparse/update.inc';
			}

			$shown++;

		}

		/* delete unused */

		if (count($ids) > 0) {
			$query = sprintf("SELECT string_id FROM strings WHERE page_id=%d AND string_id NOT IN (%s)",
				$page_id,
				join(',', $ids )
			);

			$q2 = $DB->query($query);
			if ($q2->numRows()!=0) {
				$old_ids = array();
				while ($r2 = $q2->fetchRow()) {
					$old_ids[] = $r2['string_id'];
				}
				$query = sprintf("DELETE FROM strings WHERE string_id IN (%s)",
					join(',', $old_ids )
				);
				$q2 = $DB->query($query);
				$deleted += $DB->affectedRows();

				$query = sprintf("DELETE FROM translations WHERE string_id IN (%s)",
					join(',', $old_ids )
				);
				$q2 = $DB->query($query);
			}

		}

	}		


	$query = sprintf("UPDATE pages SET db_revision='%s', cvs_revision='%s' WHERE page_id=%d",
		$new_rev,
		$new_rev,
		$page_id
	);
	$q = $DB->query($query);

}

require BABEL_TEMPLATES . '/reparse/summary.inc';
require BABEL_TEMPLATES . '/reparse/footer.inc';

if (count($errors)!=0) {
    include BABEL_TEMPLATES . '/reparse/errors.inc';
}

require $registry->get('templates', 'horde') . '/common-footer.inc';
